# -*- coding: utf-8 -*-

"""Controller API for `clusters' part.

    Functionality description:

    Let's have the main subcontroller (ClusterController) available through
    'clusters' identifier of the root controller and subcontroller handling
    related commands (ClusterCmdController) through 'clusters_cmd' identifier.

    Luci application is accessible via http://example.com. Clusters
    in the system are ClusterOne, ClusterTwo and ClusterThree.

    Accessing following URLs has following consequences (some cases omitted):

    1) <http://example.com/clusters> OR <http://example.com/clusters/>
       - displaying list of the clusters with some information about them.

    2) <http://example.com/clusters/ClusterOne>
       OR <http://example.com/clusters/ClusterOne/>
       - redirection to nodes listing related to cluster called `ClusterOne'
         (<http://example.com/clusters/ClusterOne/nodes>) because there is no
         general page for certain cluster, only a few pages with further
         information about it (and the `nodes' is probably the most important).

    3) <http://example.com/clusters/NonExistingCluster>
       OR <http://example.com/clusters/NonExistingCluster/>
       OR <http://example.com/clusters/NonExistingCluster/something_more>
       - redirection back to base `clusters' page and displaying information
         about the request of non-existing cluster.

    ---

    4) <http://example.com/clusters_cmd/create>
       - display form for cluster creation.

    5) <http://example.com/clusters_cmd/add>
       - display form for existing cluster addition.

"""

from tg import flash, request, redirect, expose, tmpl_context, app_globals, validate
from pylons.i18n import ugettext as _, lazy_ugettext as l_

from luci.lib.base import Subcontroller, SubcontrollerApplyMixin
from luci.controllers.decorators import *
from luci.lib.strings import TitleStrings
from luci.lib.helpers import urlList2String

from luci.widgets.create_cluster_form import create_cluster_form

__all__ = ['ClusterController', 'ClusterCmdController']


# Imports into module's namespace.
scheme = app_globals.scheme
cluster_scheme = app_globals.scheme.ClusterScheme 
data = app_globals.data
base2CmdCtrl = app_globals.scheme.base2CmdCtrl


#
# ITEMS LISTING.
#

class ClusterController(Subcontroller):
    """Subcontroller handling basic requests related with `clusters' part."""

    @forPageShowWithUrlCorrection('luci.templates.cluster_list')
    def default(self):
        """Handle simple cluster listing.

        E.g. <http://example.com/clusters> for situation described
        in the module's doc.

        """
        tmpl_context.title_list.append(TitleStrings.CLUSTERS)

        app_url = scheme.APP_PREFIX + u'/'

        return dict(base_url=app_url + scheme.CLUSTERS,
                    cmd_url=app_url + base2CmdCtrl(scheme.CLUSTERS),
                    apply_cmds=ClusterApplyCommands)

    @expose()
    def lookup(self, name, *args):
        """Dynamic dispatching to subcontrollers according to cluster's name.

        E.g. <http://example.com/clusters/ClusterOne> for situation described
        in the module's doc.
        
        The request is dynamically delegated to _CertainClusterController
        object.
        
        Keyword arguments:
            name    Name of the cluster to be taken as a context for
                    nested subcontrollers (NodeController, etc.).        
        
        """
        if name in data.clusters.iterkeys():
            # Following 3 lines necessary because of strange(?) behavior of TG.
            if not tmpl_context.already_used.get('cluster'):
                tmpl_context.title_list.append(TitleStrings.CERTAIN_CLUSTER \
                                               % name)
                tmpl_context.cluster_name = name
                tmpl_context.cluster_data = data.clusters[name]
                tmpl_context.cluster_url = \
                    urlList2String(scheme.APP_PREFIX, scheme.CLUSTERS, name)
                tmpl_context.already_used['cluster'] = True
            dynamic_ctrl = _CertainClusterController()
            return dynamic_ctrl, args
        else:
            msg = l_('Bad cluster name: %s') % name
            status = 'error'

        flash(msg, status=status)
        redirect(urlList2String(scheme.APP_PREFIX, scheme.CLUSTERS))


@mountSubControllers(cluster_scheme.mounts())
class _CertainClusterController(Subcontroller):
    """Subcontroller preparing subcontrollers for certain cluster."""

    @forImmediateRedirect(allowed_methods='GET', use_referrer=False)
    def default(self, *args, **kwargs):
        """Handle request for displaying details of certain cluster.
        
        E.g. <http://example.com/clusters/ClusterOne> for situation described
        in the module's doc.
        
        Redirect to `nodes' part in the context the cluster.
        
        """
        return dict(redir_target= urlList2String(tmpl_context.cluster_url,
                                                 cluster_scheme.NODES))

    # 'lookup' method not necessary, 'default' method above is enough even for
    # such cases.


#
# COMMANDS HANDLING.
#

class ClusterApplyCommands:
    """Encapsulation of available commands to apply over selected cluster(s)."""

    DELETE = 'cmd_delete'


    def _delete(which):
        clusters = app_globals.data.clusters
        msg = l_('Deleting selected cluster(s)... %s') \
                  % (', '.join(which))
        status = 'info'
        for cluster in which:
            if clusters.has_key(cluster):
                del clusters[cluster]
            else:
                # If Luci's data are consistent, this should never happen.
                msg = l_('Internal error.'),
                status = 'error'
                break
        return msg, status, False


    # Mapping.
    cmds = {DELETE: _delete}

    # Set methods as static (but only after they are referenced
    # in 'cmds' dict).
    _delete = staticmethod(_delete)


class ClusterCmdController(Subcontroller, SubcontrollerApplyMixin):
    """Subcontroller handling commands related with `clusters' part."""

    _apply_cmds = ClusterApplyCommands

    @forPageShowWithUrlCorrection('luci.templates.create')
    def create(self, name=""):
        """Display form for cluster creation."""
        return dict(cmd_url= \
                        scheme.APP_PREFIX + u'/' + base2CmdCtrl(scheme.CLUSTERS))


    @forImmediateRedirect(allowed_methods = 'GET')
    def add(self):
        """Handle addition of a new cluster."""

        flash('It should be a dialog to add new cluster here instead,'
              'but not implemented yet...', status='info')
        # following code is only my experiment, how to continue:
        #tmpl_context.form = create_fdom_add_form
        #return dict()


    @forImmediateRedirect(allowed_methods='POST')
    def cluster_create(self, **kw):
        """Handle the process of cluster creation."""

        from sys import stderr
        from luci.lib.ricci_helpers import send_batch_to_hosts
        from luci.lib.ricci_communicator import RicciCommunicator, RicciError
        import luci.lib.ricci_queries as rq

        redir_target = urlList2String(scheme.APP_PREFIX,
                                      base2CmdCtrl(scheme.CLUSTERS),
                                      'create')

        errors = []
        cluster_name = kw.get('clustername')
        enable_storage = kw.get('enable_storage')
        reboot_nodes = kw.get('reboot_nodes')
        download_pkgs = kw.get('download_pkgs')

        nodes = [
            [ kw.get('__SYSTEM0_addr'), kw.get('__SYSTEM0_passwd') ],
            [ kw.get('__SYSTEM1_addr'), kw.get('__SYSTEM1_passwd') ],
            [ kw.get('__SYSTEM2_addr'), kw.get('__SYSTEM2_passwd') ]
        ]
        node_list = [ i[0] for i in nodes ]
        stderr.write('nodes: %s\n' % str(node_list))

        if not cluster_name:
            flash(_('No cluster name was given'))
            return dict(redir_target=redir_target)

        if len(cluster_name) > 15:
            flash(_('Cluster names must be less than 16 characters long'))
            return dict(redir_target=redir_target)

        for node in nodes:
            try:
                rc = RicciCommunicator(node[0], enforce_trust=False)
            except RicciError, errmsg:
                errors.append(str(errmsg))
            else:
                rc.trust()
                rc.auth(node[1])
                if not rc.authed():
                   errors.append('Authentication to node %s failed' % node[0])
                   break
                else:
                    rc = RicciCommunicator(node[0])

                cur_cluster_name = rc.cluster_info()[0]
                if cur_cluster_name:
                    errors.append('%s is already a member of a cluster %s' \
                        % (node[0], cur_cluster_name))
                    break

        if len(errors) > 0:
            flash('The following errors occurred: %s' % str(errors))
            return dict(redir_target=redir_target)

        ret = send_batch_to_hosts(node_list, 10, rq.create_cluster,
                'rhel5', cluster_name, cluster_name,
                node_list, True, True, enable_storage, False,
                download_pkgs, None, reboot_nodes)

        base_url_str = urlList2String(scheme.APP_PREFIX, self.base_name)
        return dict(use_referrer=False, redir_target=base_url_str)
