# -*- coding: utf-8 -*-

"""Controller API for `failovers of certain cluster' part.

    Functionality description:

    Let's have the main subcontroller (FailoverController) available through
    'failovers' identifier of the `clusters' controller and subcontroller
    handling related commands (FailoverCmdController) through 'failovers_cmd'
    identifier.

    Main page of certain cluster (`ClusterOne') is accessible via
    http://example.com/clusters/ClusterOne. Failovers in the cluster are
    Failover1 and Failover2.

    Accessing following URLs has following consequences (some cases omitted):

    1) <http://example.com/clusters/ClusterOne/failovers>
       OR <http://example.com/clusters/ClusterOne/failovers/>
       - displaying list of the failovers with some information about them.

    2) <http://example.com/clusters/ClusterOne/failovers/Failover1>
       OR <http://example.com/clusters/ClusterOne/failovers/Failover1/>
       - the same list as in case 1) is displayed, but also details to failover
         `Failover1' are shown in the lower part of the page.

    3) <http://example.com/clusters/ClusterOne/failovers/NonExistingFailover>
       OR <http://example.com/clusters/ClusterOne/failovers/NonExistingFailover/>
       OR <http://example.com/clusters/ClusterOne/failovers/NonExistingFailover/something_more>
       - redirection back to base `failovers' page and displaying information
         about the request of non-existing failover.

    ---

    4) <http://example.com/clusters/ClusterOne/failovers_cmd/add>
       - display form for a failover addition.

    5) <http://example.com/clusters/ClusterOne/failovers_cmd/update_properties>
       - apply updating properties of certain failover.

    6) <http://example.com/clusters/ClusterOne/failovers_cmd/update_members>
       - apply updating members of certain failover.

    7) <http://example.com/clusters/ClusterOne/failovers_cmd/apply>
       OR <http://example.com/clusters/ClusterOne/failovers_cmd/apply?CMDID...>
       - apply a command on given failover(s); both GET and POST method
         supported.

"""
from tg import flash, redirect, expose, tmpl_context, app_globals
from pylons.i18n import ugettext as _, lazy_ugettext as l_

from luci.lib.base import Subcontroller, SubcontrollerApplyMixin
from luci.controllers.decorators import *
from luci.lib.strings import TitleStrings, VerboseStrings
from luci.lib.helpers import urlList2String

__all__ = ['FailoverController', 'FailoverCmdController']


# Imports into module's namespace.
form_utils = app_globals.form_utils
cluster_scheme = app_globals.scheme.ClusterScheme
base2CmdCtrl = app_globals.scheme.base2CmdCtrl


#
# ITEMS LISTING.
#

class FailoverController(Subcontroller):
    """Subcontroller handling basic requests related with `failovers' part."""

    @forPageShowWithUrlCorrection('luci.templates.cluster_part.failover')
    def default(self):
        """Handle simple failovers listing.
        
        E.g. <http://example.com/clusters/ClusterOne/failovers> for situation
        described in the module's doc.        
        
        """
        tmpl_context.title_list.append(TitleStrings.FAILOVERS)

        cluster_url = tmpl_context.cluster_url + u'/'

        return dict(name=None,
                    base_url=cluster_url + cluster_scheme.FAILOVERS,
                    cmd_url=cluster_url + base2CmdCtrl(cluster_scheme.FAILOVERS),
                    service_cmd_url= \
                        cluster_url + base2CmdCtrl(cluster_scheme.SERVICES),
                    apply_cmds=FailoverApplyCommands)


    @expose()
    def lookup(self, name, *args):
        """Handle failovers listing with details of certain failover.

        E.g. <http://example.com/clusters/ClusterOne/failovers/Failover1>
        for situation described in the module's doc.

        The request is dynamically delegated to _CertainFailoverController
        object.

        Keyword arguments:
            name    Name of the failover to be displayed in more detail.

        """
        # Check whether required failover exists.
        if tmpl_context.cluster_data.failovers.has_key(name):
            dynamic_ctrl = _CertainFailoverController(entity_name=name)
            return dynamic_ctrl, args
        else:
            msg = l_('Bad node name.')
            status = 'error'

        flash(msg, status=status)
        redirect(urlList2String(tmpl_context.cluster_url,
                                cluster_scheme.FAILOVERS))


class _CertainFailoverController(Subcontroller):
    """Subcontroller handling failovers listing with details of certain failover."""

    @forPageShowWithUrlCorrection('luci.templates.cluster_part.failover')
    def default(self):
        """Handle failovers listing with details of certain failover.

        E.g. <http://example.com/clusters/ClusterOne/failovers/Failover1>
        for situation described in the module's doc.
        
        """
        tmpl_context.title_list.append(TitleStrings.CERTAIN_FAILOVER \
                                       % self.entity_name)

        cluster_url = tmpl_context.cluster_url + u'/'

        return dict(name=self.entity_name,
                    base_url=cluster_url + cluster_scheme.FAILOVERS,
                    cmd_url=cluster_url + base2CmdCtrl(cluster_scheme.FAILOVERS),
                    service_cmd_url= \
                        cluster_url + base2CmdCtrl(cluster_scheme.SERVICES),
                    apply_cmds=FailoverApplyCommands)


    @expose()
    def lookup(self, *args):
        """Handle wrong URL (longer than expected) connected with certain failover.

        E.g. <http://example.com/clusters/ClusterOne/failovers/Failover1/something>
        for situation described in the module's doc.

        """
        redirect(urlList2String(tmpl_context.cluster_url,
                                cluster_scheme.FAILOVERS,
                                self.entity_name))


#
# COMMANDS HANDLING.
#

class FailoverApplyCommands:
    """Encapsulation of available commands to apply over selected failover(s)."""

    DELETE = 'cmd_delete'


    def _delete(which):
        failovers = tmpl_context.cluster_data.failovers
        msg = l_('Deleting selected failover(s)... %s') \
                  % (', '.join(which))
        status = 'info'
        for failover in which:
            if failovers.has_key(failover):
                del failovers[failover]
            else:
                # If Luci's data are consistent, this should never happen.
                msg = l_('Internal error.'),
                status = 'error'
                break
        return msg, status, False


    # Mapping.
    cmds = {DELETE: _delete}

    # Set methods as static (but only after they are referenced
    # in 'cmds' dict).
    _delete = staticmethod(_delete)


class FailoverCmdController(Subcontroller, SubcontrollerApplyMixin):
    """Subcontroller handling commands related with `failovers' part."""

    _apply_cmds = FailoverApplyCommands

    @forImmediateRedirect(allowed_methods = ('GET', 'POST'))
    def default(self, *args, **kwargs):
        """Handle requests not connected with any of defined method."""
        
        flash(VerboseStrings.BAD_COMMAND_REQUEST, status='error')
        
        base_url_str = urlList2String(tmpl_context.cluster_url, base_name)
        return dict(redir_target=base_url_str)


    @forImmediateRedirect(allowed_methods = 'GET')
    def add(self):
        """Handle creation of a new failover domain."""

        flash('It should be a dialog to add new failover domain here instead, '
              'but not implemented yet...', status='info')
        # following code is only my experiment, how to continue:
        #tmpl_context.form = create_fdom_add_form
        #return dict()


    @forImmediateRedirect(allowed_methods = 'POST')
    def update_properties(self, name=None, **kwargs):
        """Handle changes in a failover domain -- properties.

        Keyword arguments:
            name    Name of the failover, which attributes are being changed.
            kwargs  Dict of attributes that are active.

        """
        if not name:
            flash(_('Internal error.'), status='error')
        else:
            failovers = tmpl_context.cluster_data.failovers
            failover = form_utils.id2String(name)
            if failovers.has_key(failover):
                for attr in ['prioritized', 'restricted', 'failback']:
                    if kwargs.has_key(attr):
                        failovers[failover][attr] = True
                    else:
                        failovers[failover][attr] = False
                flash(_('Updating properties for %s') % failover, status='info')
            else:
                # If the luci's data are consistent this should never happen.
                flash(_('Internal error.'), status='error')


    @forImmediateRedirect(allowed_methods = 'POST')
    def update_members(self, name=None, **kwargs):
        """Handle changes in a failover domain -- member nodes.

        Keyword arguments:
            name    Name of the failover, which members are being changed.
            kwargs  Dict of values of members.

        """
        if not name:
            flash(_('Internal error.'), status='error')
        else:
            failovers = tmpl_context.cluster_data.failovers
            failover = form_utils.id2String(name)
            if failovers.has_key(failover):
                # Clear the dictionary of member nodes and start from scratch.
                failovers[failover]['nodes'].clear()
                nodes = \
                    map(lambda node_id: (node_id, form_utils.id2String(node_id)),
                        map(lambda node_id: node_id.replace(u'.check', u''),
                            filter(lambda param: param.endswith(u'.check'),
                                   kwargs.iterkeys())))
                flash(_('Updating members for %s') % failover, status='info')
                for node_id, node in nodes:
                    if tmpl_context.cluster_data.nodes.has_key(node):
                        failovers[failover]['nodes'][node] = \
                            kwargs.get(node_id + '.priority', 0)
                    else:
                        # If the luci's data are consistent this should never
                        # happen.
                        flash(_('Internal error'), status='error')


