# -*- coding: utf-8 -*-

"""Controller API for `fences of certain cluster' part.

    Functionality description:

    Let's have the main subcontroller (FenceController) available through
    'fences' identifier of the `clusters' controller and subcontroller handling
    related commands (FenceCmdController) through 'fences_cmd' identifier.

    Main page of certain cluster (`ClusterOne') is accessible via
    http://example.com/clusters/ClusterOne. Fences in the cluster are
    FenceA and FenceB.

    Accessing following URLs has following consequences (some cases omitted):

    1) <http://example.com/clusters/ClusterOne/fences>
       OR <http://example.com/clusters/ClusterOne/fences/>
       - displaying list of the fences with some information about them.

    2) <http://example.com/clusters/ClusterOne/fences/FenceA>
       OR <http://example.com/clusters/ClusterOne/fences/FenceA/>
       - the same list as in case 1) is displayed, but also details to fence
         `FenceA' are shown in the lower part of the page.

    3) <http://example.com/clusters/ClusterOne/fences/NonExistingFence>
       OR <http://example.com/clusters/ClusterOne/fences/NonExistingFence/>
       OR <http://example.com/clusters/ClusterOne/fences/NonExistingFence/something_more>
       - redirection back to base `fences' page and displaying information
         about the request of non-existing fence.

    ---

    4) <http://example.com/clusters/ClusterOne/fences_cmd/add>
       - display form for a fence addition.

    5) <http://example.com/clusters/ClusterOne/fences_cmd/apply>
       OR <http://example.com/clusters/ClusterOne/fences_cmd/apply?CMDID...>
       - apply a command on given node(s); both GET and POST method supported.

"""
from tg import flash, redirect, expose, tmpl_context, app_globals
from pylons.i18n import ugettext as _, lazy_ugettext as l_

from luci.lib.base import Subcontroller, SubcontrollerApplyMixin
from luci.controllers.decorators import *
from luci.lib.strings import TitleStrings, VerboseStrings
from luci.lib.helpers import urlList2String


__all__ = ['FenceController', 'FenceCmdController']


# Imports into module's namespace.
form_utils = app_globals.form_utils
cluster_scheme = app_globals.scheme.ClusterScheme
base2CmdCtrl = app_globals.scheme.base2CmdCtrl


#
# ITEMS LISTING.
#

class FenceController(Subcontroller):
    """Subcontroller handling basic requests related with `fences' part."""

    @forPageShowWithUrlCorrection('luci.templates.cluster_part.fence')
    def default(self):
        """Handle simple fences listing.

        E.g. <http://example.com/clusters/ClusterOne/fences> for situation
        described in the module's doc.

        """
        tmpl_context.title_list.append(TitleStrings.FENCES)

        cluster_url = tmpl_context.cluster_url + u'/'

        return dict(name=None,
                    base_url=cluster_url + cluster_scheme.FENCES,
                    cmd_url=cluster_url + base2CmdCtrl(cluster_scheme.FENCES),
                    node_base_url=cluster_url + cluster_scheme.NODES,
                    apply_cmds=FenceApplyCommands)


    @expose()
    def lookup(self, name, *args):
        """Handle fences listing with details of certain fence.

        E.g. <http://example.com/clusters/ClusterOne/fences/FenceA>
        for situation described in the module's doc.

        The request is dynamically delegated to _CertainFenceController object.

        Keyword arguments:
            name    Name of the fence to be displayed in more detail.

        """
        # Check whether required fence exists.
        if tmpl_context.cluster_data.fences.has_key(name):
            dynamic_ctrl = _CertainFenceController(entity_name=name)
            return dynamic_ctrl, args
        else:
            msg = l_('Bad fence name.')
            status = 'error'

        flash(msg, status=status)
        redirect(urlList2String(tmpl_context.cluster_url,
                                cluster_scheme.FENCES))


class _CertainFenceController(Subcontroller):
    """Subcontroller handling fences listing with details of certain fence."""

    @forPageShowWithUrlCorrection('luci.templates.cluster_part.fence')
    def default(self):
        """Handle fences listing with details of certain fence.

        E.g. <http://example.com/clusters/ClusterOne/fences/FenceA>
        for situation described in the module's doc.

        """
        tmpl_context.title_list.append(TitleStrings.CERTAIN_FENCE \
                                       % self.entity_name)

        cluster_url = tmpl_context.cluster_url + u'/'

        return dict(name=self.entity_name,
                    base_url=cluster_url + cluster_scheme.FENCES,
                    cmd_url=cluster_url + base2CmdCtrl(cluster_scheme.FENCES),
                    node_base_url=cluster_url + cluster_scheme.NODES,
                    apply_cmds=FenceApplyCommands)


    @expose()
    def lookup(self, *args):
        """Handle wrong URL (longer than expected) connected with certain node.

        E.g. <http://example.com/clusters/ClusterOne/nodes/NodeAlpha/something>
        for situation described in the module's doc.

        """
        redirect(urlList2String(tmpl_context.cluster_url, cluster_scheme.FENCES,
                                self.entity_name))


#
# COMMANDS HANDLING.
#

class FenceApplyCommands:
    """Encapsulation of available commands to apply over selected fence(s)."""

    UPDATE = 'cmd_update'
    DELETE = 'cmd_delete'


    def _update(which):
        msg = l_('Updating selected fence(s)... %s') \
                % (", ".join(which))
        status = 'info'
        return msg, status


    def _delete(which):
        fences = tmpl_context.cluster_data.fences
        msg = l_('Deleting selected fence(s)... %s') \
                  % (', '.join(which))
        status = 'info'
        for fence in which:
            if fences.has_key(fence):
                del fences[fence]
            else:
                # If Luci's data are consistent, this should never happen.
                msg = l_('Internal error.'),
                status = 'error'
                break
        return msg, status, False


    # Mapping.
    cmds = {UPDATE: _update,
            DELETE: _delete}

    # Set methods as static (but only after they are referenced
    # in 'cmds' dict).
    _update = staticmethod(_update)
    _delete = staticmethod(_delete)


class FenceCmdController(Subcontroller, SubcontrollerApplyMixin):
    """Subcontroller handling commands related with `fences' part."""

    _apply_cmds = FenceApplyCommands

    @forImmediateRedirect(allowed_methods = ('GET', 'POST'))
    def default(self, *args, **kwargs):
        """Handle requests not connected with any of defined method."""

        flash(VerboseStrings.BAD_COMMAND_REQUEST, status='error')

        base_url_str = urlList2String(tmpl_context.cluster_url, self.base_name)
        return dict(redir_target=base_url_str)


    @forImmediateRedirect(allowed_methods = 'GET')
    def add(self):
        """Handle creation of a new fence."""

        flash('It should be a dialog to add new fence here instead, but not'
              ' implemented yet...', status='info')


#    @forImmediateRedirect(allowed_methods = 'GET')
#    def manage(self):
#        """Handle the nodes management."""
#
#        flash('Demo of managing nodes...', status='info')
