# -*- coding: utf-8 -*-

"""Controller API for `services of certain cluster' part.

    Functionality description:

    Let's have the main subcontroller (ServiceController) available through
    'services' identifier of the `clusters' controller and subcontroller
    handling related commands (ServiceCmdController) through 'services_cmd'
    identifier.

    Main page of certain cluster (`ClusterOne') is accessible via
    http://example.com/clusters/ClusterOne. Services in the cluster are
    ServiceX, ServiceY and ServiceZ.

    Accessing following URLs has following consequences (some cases omitted):

    1) <http://example.com/clusters/ClusterOne/services>
       OR <http://example.com/clusters/ClusterOne/services/>
       - displaying list of the services with some information about them.

    2) <http://example.com/clusters/ClusterOne/services/ServiceX>
       OR <http://example.com/clusters/ClusterOne/services/ServiceX/>
       - the same list as in case 1) is displayed, but also details to service
         `ServiceX' are shown in the lower part of the page.

    3) <http://example.com/clusters/ClusterOne/services/NonExistingService>
       OR <http://example.com/clusters/ClusterOne/services/NonExistingService/>
       OR <http://example.com/clusters/ClusterOne/services/NonExistingService/something_more>
       - redirection back to base `services' page and displaying information
         about the request of non-existing service.

    ---

    4) <http://example.com/clusters/ClusterOne/services_cmd/add>
       - display form for a node addition to cluster.

    5) <http://example.com/clusters/ClusterOne/services_cmd/apply>
       OR <http://example.com/clusters/ClusterOne/services_cmd/apply?CMDID...>
       - apply a command on given service(s); both GET and POST method supported.

"""
from tg import flash, redirect, expose, tmpl_context, app_globals
from pylons.i18n import ugettext as _, lazy_ugettext as l_

from luci.lib.base import Subcontroller, SubcontrollerApplyMixin
from luci.controllers.decorators import *
from luci.lib.strings import TitleStrings, VerboseStrings
from luci.lib.helpers import urlList2String

__all__ = ['ServiceController', 'ServiceCmdController']


# Imports into module's namespace.
form_utils = app_globals.form_utils
cluster_scheme = app_globals.scheme.ClusterScheme
base2CmdCtrl = app_globals.scheme.base2CmdCtrl


#
# ITEMS LISTING.
#

class ServiceController(Subcontroller):
    """Subcontroller handling basic requests related with `services' part."""

    @forPageShowWithUrlCorrection('luci.templates.cluster_part.service')
    def default(self):
        """Handle simple services listing.

        E.g. <http://example.com/clusters/ClusterOne/services> for situation
        described in the module's doc.

        """
        tmpl_context.title_list.append(TitleStrings.SERVICES)

        cluster_url = tmpl_context.cluster_url + u'/'

        return dict(name=None,
                    base_url=cluster_url + cluster_scheme.SERVICES,
                    cmd_url=cluster_url + base2CmdCtrl(cluster_scheme.SERVICES),
                    apply_cmds=ServiceApplyCommands)


    @expose()
    def lookup(self, name, *args):
        """Handle service listing with details of certain service.

        E.g. <http://example.com/clusters/ClusterOne/services/ServiceX>
        for situation described in the module's doc.

        The request is dynamically delegated to _CertainServiceController
        object.

        Keyword arguments:
            name    Name of the service to be displayed in more detail.

        """
        # Check whether required service exists.
        if tmpl_context.cluster_data.services.has_key(name):
            dynamic_ctrl = _CertainServiceController(entity_name=name)
            return dynamic_ctrl, args
        else:
            msg = l_('Bad service name.')
            status = 'error'

        flash(msg, status=status)
        redirect(urlList2String(tmpl_context.cluster_url, cluster_scheme.SERVICES))


class _CertainServiceController(Subcontroller):
    """Subcontroller handling services listing with details of certain service."""

    @forPageShowWithUrlCorrection('luci.templates.cluster_part.service')
    def default(self):
        """Handle services listing with details of certain service.

        E.g. <http://example.com/clusters/ClusterOne/services/ServiceX>
        for situation described in the module's doc.

        """
        tmpl_context.title_list.append(TitleStrings.CERTAIN_SERVICE \
                                       % self.entity_name)

        cluster_url = tmpl_context.cluster_url + u'/'

        return dict(name=self.entity_name,
                    base_url=cluster_url + cluster_scheme.SERVICES,
                    cmd_url=cluster_url + base2CmdCtrl(cluster_scheme.SERVICES),
                    apply_cmds=ServiceApplyCommands)


    @expose()
    def lookup(self, *args):
        """Handle wrong URL (longer than expected) connected with certain service.

        E.g. <http://example.com/clusters/ClusterOne/services/ServiceX/something>
        for situation described in the module's doc.

        """
        redirect(urlList2String(tmpl_context.cluster_url,
                                cluster_scheme.SERVICES, self.node_name))


#
# COMMANDS HANDLING.
#

class ServiceApplyCommands:
    """Encapsulation of available commands to apply over selected service(s)."""

    REBOOT = 'cmd_reboot'
    DELETE = 'cmd_delete'


    def _reboot(which):
        msg = l_('Rebooting selected service(s)... %s') \
              % (", ".join(which))
        status = 'info'
        return msg, status


    def _delete(which):
        services = tmpl_context.cluster_data.services
        msg = l_('Deleting selected service(s)... %s') \
                  % (', '.join(which))
        status = 'info'
        for service in which:
            if services.has_key(service):
                del services[service]
            else:
                # If Luci's data are consistent, this should never happen.
                msg = l_('Internal error.'),
                status = 'error'
                break
        return msg, status, False


    # Mapping.
    cmds = {REBOOT: _reboot,
            DELETE: _delete}

    # Set methods as static (but only after they are referenced
    # in 'cmds' dict).
    _reboot = staticmethod(_reboot)
    _delete = staticmethod(_delete)


class ServiceCmdController(Subcontroller, SubcontrollerApplyMixin):
    """Subcontroller handling commands related with `services' part."""

    _apply_cmds = ServiceApplyCommands

    @forImmediateRedirect(allowed_methods = ('GET', 'POST'))
    def default(self, *args, **kwargs):
        """Handle requests not connected with any of defined method."""

        flash(VerboseStrings.BAD_COMMAND_REQUEST, status='error')

        base_url_str = urlList2String(tmpl_context.cluster_url, self.base_name)
        return dict(redir_target=base_url_str)


    @forImmediateRedirect(allowed_methods = 'GET')
    def add(self, node=None, failover=None):
        """Handle creation of a new service."""

        msg = 'DEMO: Adding a service'
        if node!= None:
            msg += ', for node %s' % node
        if failover != None:
            msg += ', for failover %s' % failover

        flash(msg, status='info')

        # following code is only my experiment, how to continue:
        #tmpl_context.form = create_fdom_add_form
        #return dict()

        base_url_str = urlList2String(tmpl_context.cluster_url, self.base_name)
        return dict(redir_target=base_url_str)

