# -*- coding: utf-8 -*-

"""Decorators used within the application's controllers."""

from tg import url, redirect, request, flash, expose, app_globals
from pylons.i18n import ugettext as _, lazy_ugettext as l_

from luci.lib.helpers import urlList2String

__all__ = ['mountSubControllers',
           'forPageShowWithUrlCorrection',
           'forPageShowOnly',
           'forImmediateRedirect']


# Imports into module's namespace.
APP_PREFIX = app_globals.scheme.APP_PREFIX
base2CmdCtrl = app_globals.scheme.base2CmdCtrl


def mountSubControllers(ctrls_dict=None, **ctrls_kwargs):
    """Class decorator serving as an elegant way to `mount' subcontrollers.

    The key of each keyword argument (or alternatively taken from 'ctrls_dict'
    argument, which can be used to pass the same in the form of single dict)
    is used as an identifier where to `mount' such object of the subcontroller.

    A bit more advanced use of this subcontroller -- let's have a pair of
    related subcontrollers, base one for general items listing (e.g. ItemController)
    and the other for applying command over set of selected items and similar
    actions (e.g. ItemCmdController). It is useful to let them `mounted'
    on related identifiers.
    
    Notice: subcontrollers mounted in this fashion should be inherited from
            'Subcontroller' class as it is prepared to deal with arguments
            passed on object creation (see luci.lib.base).    
    
    Let's have this example of use:
    ---
    @mountLuciSubControllers(items=(ItemController, ItemCmdController))
    class RootController(BaseController):
        ...
    ---
    Then it's equivalent with following commands sequence:
    ---
    class RootController(BaseController):
        items = ItemController()
        items_cmd = ItemCmdController(base_name='items')
        ...
    ---
    Notice: '_cmd' suffix is default suffix for `commands subcontroller'
            as given by 'base2CmdCtrl()' (see luci.controllers.scheme).

    Keyword arguments:
        ctrls_dict         Same purpose as ctrls_kwargs argument, but for
                           passing subcontrollers in the form of a single dict.
        ctrls_kwargs       Definition of subcontrollers, see examples above.

    """
    def decorator(cls):
        """Class binding."""
        # Take all specified subcontrollers and `mount' them to the class.
        if ctrls_dict and type(ctrls_dict is dict):
            ctrls_kwargs.update(ctrls_dict)
        for base_name, controller in ctrls_kwargs.iteritems():
            # Branch according to whether a pair of controllers is used
            # instead of a single controller.
            if type(controller) in (tuple, list) and len(controller) == 2:
                # 1) Pair of controllers.
                cmd_name = base2CmdCtrl(base_name)
                setattr(cls, base_name, controller[0]())
                setattr(cls, cmd_name, controller[1](base_name=base_name))
            else:
                # 2) Single controller.
                if type(controller) in (tuple, list):
                    controller = controller[0]
                setattr(cls, base_name, controller())

        # Decorator's retval is the value returned by the wrapper.
        return cls

    # Retval is the value returned by the decorator.
    return decorator



ATTR_REDIR_TARGET = 'REDIR_TARGET'


def forPageShowOnly(template='', allowed_methods=('GET', 'POST'),
                    redirect='', use_referrer=True):
    """Decorator to show page incl. check of the request's method.

    Keyword arguments:
        template         Which template to internally use with @expose().
        allowed_methods  Tuple listing all supported methods (e.g. GET or POST).
        redir_target     Where to redirect on failure. If omitted, it can
                         still be specified through 'REDIR_TARGET' attribute
                         (ATTR_REDIR_TARGET) of the object that has bound
                         decorated method. If nothing specified at all,
                         LUCI_APP_PREFIX is used.
        use_referrer     On redirect call, try to use referrer first.

    """
    if type(allowed_methods) not in (tuple, list):
        allowed_methods = [allowed_methods]

    def decorator(fn):
        """Function (method) binding"""
        @expose(template)
        def wrapper(self, *args, **kwargs):
            """Arguments binding, decorated function (method) call."""
            if request.method not in allowed_methods:
                flash(_('Unsupported method of the request.'), status='error')
                redir_url = redir_target
                if use_referrer and request.referrer:
                    redir_url = request.referrer
                elif not redir_url:
                    redir_url = getattr(fn.__self__, ATTR_REDIR_TARGET,
                                        LUCI_APP_PREFIX)
                redirect(redir_url)
            else:
                return fn(self, *args, **kwargs)

        # Decorator's retval is the value returned by the wrapper.
        return wrapper

    # Retval is the value returned by the decorator.
    return decorator


def forPageShowWithUrlCorrection(template=''):
    """Decorator to show page incl. stripping off unsupported kwargs from URL.

    Obviously, the only supported method is GET.

    Keyword arguments:
        template    Which template to internally use with @expose().

    """
    def decorator(fn):
        """Function (method) binding"""
        @expose(template)
        def wrapper(self, *args, **kwargs):
            """Arguments binding, decorated function (method) call."""
            if request.method != 'GET':
                flash(_('Unsupported method of the request.'), status='error')
                redirect(request.path, **kwargs)
            else:
                redir = False
                # Get all the keyword arguments of original function.
                fn_kwargs = fn.func_code.co_varnames[1:fn.func_code.co_argcount]
                for kwarg in kwargs.keys():
                    # If some keyword arg that is not supported by original fun.
                    # was found, remove it from keyword args dict and set
                    # the redirect flag for redirection to 'purified URL'.
                    if not kwarg in fn_kwargs:
                        del kwargs[kwarg]
                        redir = True
                if redir:
                    redirect(request.path, **kwargs)
                else:
                    return fn(self, **kwargs)

        # Decorator's retval is the value returned by the wrapper.
        return wrapper

    # Retval is the value returned by the decorator.
    return decorator


def forImmediateRedirect(**outer_kwargs):
    """Decorator to redirect immediately after handling the request.

    Check of the request's method is applied.

    Keyword arguments (not stated directly as kwargs are better in this case):
        allowed_methods  Tuple listing all supported methods (e.g. GET or POST).
                         Default: ('GET', 'POST')
        redir_target     Where to redirect. If omitted, it can still be
                         specified through 'REDIR_TARGET' attribute
                         (ATTR_REDIR_TARGET) of the object that has bound
                         decorated method. If nothing specified at all,
                         LUCI_APP_PREFIX is used.
                         Default: ''
        use_referrer     On redirect call, try to use referrer first.
                         Default: True

    Values of these keyword arguments (excluding allowed_methods) can be changed
    additionally by the decorated method, according to its return value, which
    has to be dictionary (e.g. 'dict(use_referrer=False, redir_target='/foo')').

    """
    allowed_methods = outer_kwargs.get('allowed_methods', ('GET', 'POST'))
    if type(allowed_methods) not in (tuple, list):
        allowed_methods = [allowed_methods]

    def decorator(fn):
        """Function (method) binding."""
        @expose()
        def wrapper(self, *args, **kwargs):
            """Arguments binding, decorated function (method) call."""
            inner_kwargs = outer_kwargs.copy()
            if request.method not in allowed_methods:
                flash(_('Unsupported method of the request.'), status='error')
            else:
                override_kwargs = fn(self, *args, **kwargs)
                # If retval of decorated function is dict, override original
                # arguments with values from this dict.
                if type(override_kwargs) is dict:
                    inner_kwargs.update(override_kwargs)
            redir_url = inner_kwargs.get('redir_target', '')
            if inner_kwargs.get('use_referrer', True) and request.referrer:
                redir_url = request.referrer
            elif not redir_url:
                redir_url = getattr(self, ATTR_REDIR_TARGET, LUCI_APP_PREFIX)
            redirect(redir_url)

        # Decorator's retval is the value returned by the wrapper (no retval).
        return wrapper

    # Retval is the value returned by the decorator.
    return decorator

