# -*- coding: utf-8 -*-

"""Utilities used in connection to HTML forms."""

from base64 import b64encode, b64decode
from string import maketrans

__all__ = ['FormUtils']


class FormUtils:

    STARTING_CHAR = u'L'  # First character has to be [A-Za-z].
    
    B64_ORIG_CHARS = u'=+/'
    B64_CONV_CHARS = u'-_:'

    TRANS_TABLE_TO_CONV = maketrans(B64_ORIG_CHARS, B64_CONV_CHARS)
    TRANS_TABLE_FROM_CONV = maketrans(B64_CONV_CHARS, B64_ORIG_CHARS)


    @classmethod
    def string2Id(cls, s):
        """Convert the string to the form usable as an element's id or name.
    
        'Usable' means that it conforms (X)HTML requirements for the value
        of 'id' and 'name' attribute of an element.
    
        The implementation uses base64 encoding with some other additions;
        references:
            http://www.w3.org/TR/xhtml1/#C_8
            http://www.w3.org/TR/html4/types.html#h-6.2
            http://tools.ietf.org/html/rfc3548.html
    
        Keyword arguments:
            s    String to convert.
    
        """
        encoded = b64encode(s.encode('utf-8'))
        return cls.STARTING_CHAR + encoded.translate(cls.TRANS_TABLE_TO_CONV)
    

    @classmethod
    def id2String(cls, s):
        """Receive the string previously encoded by string2id().
    
        Keyword arguments:
            s    String to convert back to human readable format.
        """
        back = b64decode(str(s)[1:].translate(cls.TRANS_TABLE_FROM_CONV))
        return back.decode('utf-8')
    

    @staticmethod
    def compactString(s, maxlong, append='...'):
        """For string longer than maxlong, shorten it and append e.g. '...'.
    
        Keyword arguments:
            s        String to be `compacted'.
            maxlong  Maximum tolerated length of the input string.
            append   What to append to the `compacted' string.
        
        Return value:
            `Compacted' string if the input one was longer than maxlong,
            original string otherwise.
    
        """
        if len(s) > maxlong:
            return s[:maxlong] + append
        else:
            return s
